#include "pbopack.h"
#include <windows.h>
#include <stdio.h>
#include <direct.h>
#include <io.h>



extern   void     PrintConsole(const char *str,...);




PboEntry::PboEntry()
{
   filename[0]=0;
   size=0;
   memset(unknown,0,16);
}

int      PboEntry::Read(FILE *f)
{   
   while(1)
   {
      char c=fgetc(f);

      if (c==char(0))
         break;         

      filename+=c;
   }

   fread(unknown,1,16,f);

   fread(&size,1,4,f);

   if (size==0)
      return 0;

   return 1;
}


int      PboEntry::Write(FILE *f)
{
   fwrite(filename.c_str(),1,filename.size(),f);

   fputc(char(0),f);

   fwrite(unknown,1,16,f);

   fwrite(&size,1,4,f);

   return 1;
}



PboPack::PboPack()
{
}

PboPack::~PboPack()
{  
   for (PboEntryVector::iterator i=Entry.begin();i<Entry.end();i++)
   {
      delete (*i);
   }
}

void  PboPack::CreateDirs(const string &s)
{
   char curdir[4096];

   GetCurrentDirectory(4096, curdir);
   
   string   ns = s;
   string   ns2 = s;

   while(1)
   {
      int   l = ns.find('\\');

      if (l==-1)
      {
         l = ns.find('/');

         if (l==-1)
            break;
      }        
   
      ns2.erase(0,l+1);
      ns.erase(l,ns.size());

      mkdir(ns.c_str());

      chdir(ns.c_str());

      ns=ns2;
   }

   SetCurrentDirectory(curdir);
}

int   PboPack::Extract(const char *name)
{
   FILE  *f=fopen(name,"rb");

   PrintConsole("Extracting file %s...\n",name);

   if (!f)
      return 0;

   while(1)
   {
      PboEntry *p=new PboEntry;

      int ret = p->Read(f);

      Entry.push_back(p);
      
      if (!ret)
         break;
   }

   string   dir=name;
   
   dir.erase(dir.size()-4);
   
   
   mkdir(dir.c_str());   

   for (PboEntryVector::iterator i=Entry.begin();i<Entry.end();i++)
   {
      PboEntry *p=(*i);

      if (p->size==0)
         continue;

      string fname=dir + string("\\") + p->filename;

      chdir(dir.c_str());

      CreateDirs(p->filename);

      chdir("..");

      OutputDebugString(string(fname + "\n").c_str());

      PrintConsole("Extracting %s (%d bytes)\n",p->filename.c_str(),p->size);

      FILE  *fo=fopen(fname.c_str(),"wb");
      
      char  *buffer=new char[p->size];

      fread(buffer,1,p->size,f);

      fwrite(buffer,1,p->size,fo);

      delete[]buffer;

      fclose(fo);
   }
   
   fclose(f);

   PrintConsole("All done!\n");

   return 1;   
}


void  PboPack::CompressFile(const char *fn)
{

   string   fname=fn;

   fname.erase(0,CurPath.size()+1);

   FILE  *f=fopen(fn,"rb");

   PboEntry *p=new PboEntry;

   p->filename=fname;

   p->size=filelength(fileno(f));


   PrintConsole("Compressing %s (%d bytes)\n",p->filename.c_str(),p->size);

   
   for (int i=0;i<p->size;i++)
   {
      Data.push_back(fgetc(f));
   }

   fclose(f);

   Entry.push_back(p);
}

void  PboPack::CompressDir()
{
   WIN32_FIND_DATA   Find;
   HANDLE            found = FindFirstFile( "*.*" , &Find );
   // check if any file found
   if (found == INVALID_HANDLE_VALUE) return;
   //
   char buf[4096];

   GetCurrentDirectory(4096,buf);
   
   string curpath=buf;

   while( 1 )
   {
      if( !(Find.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) )
      {
         CompressFile(string(curpath + string("\\") + Find.cFileName).c_str());
      }
      //
      if( FindNextFile( found , &Find ) == 0 ) 
      {
         FindClose(found);
         break;
      }

   }
}


void  PboPack::CompressTree()
{
   WIN32_FIND_DATA   Find;
   HANDLE            found = FindFirstFile( "*.*" , &Find );
   // check if any file found
   if (found == INVALID_HANDLE_VALUE) return;
   //
   CompressDir();
   //
   while( 1 )
   {
      if( Find.cFileName[0] != '.' )
      {
         if( Find.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
         {
            chdir( Find.cFileName );
            //
            CompressTree( );
            //
            chdir( ".." );
         }
      }
      //
      if( FindNextFile( found , &Find ) == 0 ) 
      {
         FindClose(found);
         break;
      }
   }
}


int   PboPack::Compress(const char *outfile, const char *path)
{  
   PrintConsole("Compressing path %s into %s...\n",path, outfile);

   char buf[256];
   GetCurrentDirectory(256,buf);

   CurPath=path;

   chdir(path);

   CompressTree();

   chdir(buf);

   PboEntry *ep=new PboEntry;

   Entry.push_back(ep);

   PrintConsole("OK!\n");

   PrintConsole("Saving %s...\n",outfile);

   FILE  *f=fopen(outfile,"wb");

   for (PboEntryVector::iterator i=Entry.begin();i<Entry.end();i++)
   {
      PboEntry *p=(*i);

      p->Write(f);
   }

   fwrite(&(*Data.begin()),1,Data.size(),f);

   const char  *cr=" ** PboTool (c)2002 JaZzBrE ** ";

   fwrite(cr,1,strlen(cr),f);

   fclose(f);

   PrintConsole("All done!\n");

   return 1;
}
